@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (0, 3, 0)
-CODENAME = "docuparty"
-BUILD_DT = (2020, 5, 6)
+VERSION = (0, 5, 6)
+CODENAME = "fuse jelly"
+BUILD_DT = (2020, 11, 29)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
